# -*- coding: utf-8 -*-
'''
Author: David A. Schulenberg
Email: schulenberg@aept.rub.de

This script is part of the research paper "Mode transition induced by gas heating along
the discharge channel in capacitively coupled atmospheric pressure micro plasma jets"

This script is licensed under the MIT License.
See the file "LICENSE.txt" for the full license text.
'''

import re
import os
import numpy as np
import matplotlib.pyplot as plt

plt.rcParams['font.family'] = 'Times New Roman'

def buildpath1(foldername):
    base_path = r'..\data_upgraded\fig_10'
    filename = 'abs.dat'
    full_path = base_path + '\\' + foldername + '\\' + filename
    return full_path

def buildpath2(foldername):
    base_path = r'..\data_upgraded\fig_10'
    filename = 'met_state_tx.dat'
    full_path = base_path + '\\' + foldername + '\\' + filename
    return full_path


def readfiles(file_path, file_path_met):
    particle_type = []
    values = []
    with open(file_path, "r") as file:
        lines = file.readlines()[39:45]
        for line in lines:
            # Split the line into characters and number using regex:
            # first group matches anything (.*?)
            # second group captures a floating point number, also written in scientific notation:
            match = re.match(r"^(.*?)([-+]?[0-9]*\.?[0-9]+([eE][-+]?[0-9]+)?)$", line.strip())
            if match:
                particle_type.append(match.group(1)[:-3])
                values.append(float(match.group(2)))
    
    particle_type.append('He*')
    values.append(np.mean(np.loadtxt(file_path_met)))
    
    values_array = np.array(values)
    file_contents = [particle_type, values_array]
    table = [file_path.split(os.sep)[-2], file_contents]
    return table


def getparticledensities(powertables, row):
    particle_density_table = []
    for table in powertables:
        particle_density = table[1][1][row]
        particle_density_table.append(particle_density)
    
    particle_density_table = np.array(particle_density_table)
    return particle_density_table


list_of_folders_1 = ['david_1f_600vpp_05n2_300k',
                     'david_1f_600vpp_05n2_325k',
                     'david_1f_600vpp_05n2_350k',
                     'david_1f_600vpp_05n2_375k',
                     'david_1f_600vpp_05n2_400k', # up tp here 0.5 sccm N2
                     'david_1f_600vpp_1n2_300k',
                     'david_1f_600vpp_1n2_325k',
                     'david_1f_600vpp_1n2_350k',
                     'david_1f_600vpp_1n2_375k',
                     'david_1f_600vpp_1n2_400k', # up tp here 1 sccm N2
                     'david_1f_600vpp_2n2_300k',
                     'david_1f_600vpp_2n2_325k',
                     'david_1f_600vpp_2n2_350k',
                     'david_1f_600vpp_2n2_375k',
                     'david_1f_600vpp_2n2_400k', # up tp here 2 sccm N2
                     ]


#create list of full paths
filepaths = []
filepaths_met = []
for folder in list_of_folders_1:
    filepaths.append(buildpath1(folder))
    filepaths_met.append(buildpath2(folder))


#read files from all the folders
densitytables = []
for i, path in enumerate(filepaths):
    densitytables.append(readfiles(path, filepaths_met[i]))


#extract and order electron densities
electron_density_table = []
temp_list = []
for table in densitytables:
    foldername = table[0]
    electron_density = table[1][1][5]
    electron_density_table.append(electron_density)
    temp_list.append(int(foldername[-4:-1]))

electron_density_table = np.array(electron_density_table)
temp_array = np.array(temp_list)


#extract and order He+ ion density values
He_p_density_table = getparticledensities(densitytables, 0)

#extract and order He+ ion density values
He2_p_density_table = getparticledensities(densitytables, 1)

#extract and order He+ ion density values
N2_p_density_table = getparticledensities(densitytables, 2)

#extract and order He+ ion density values
N4_p_density_table = getparticledensities(densitytables, 3)

#extract and order He+ ion density values
He_met_density_table = getparticledensities(densitytables, 6)


ion_density_sum = He_p_density_table + He2_p_density_table + N2_p_density_table + N4_p_density_table

#################################################################################################
######################## Plot elastic collision power densities #################################
#################################################################################################

fig, axarr = plt.subplots(3, 1, dpi=200, figsize=(4, 10))  # Adjust the figsize for your desired plot size

# First plot, #1000:0.5
ax = axarr[0]
ax.plot(temp_array[ 0: 5], He_p_density_table[ 0: 5], marker='s', linewidth=0.5, label='He$^\mathregular{+}$')
ax.plot(temp_array[ 0: 5], He2_p_density_table[ 0: 5], marker='s', linewidth=0.5, label='He$_\mathregular{2}^\mathregular{+}$')
ax.plot(temp_array[ 0: 5], N2_p_density_table[ 0: 5], marker='s', linewidth=0.5, label='N$_\mathregular{2}^\mathregular{+}$')
ax.plot(temp_array[ 0: 5], N4_p_density_table[ 0: 5], marker='s', linewidth=0.5, label='N$_\mathregular{4}^\mathregular{+}$')

ax.plot(temp_array[ 0: 5], electron_density_table[ 0: 5], marker='|', linewidth=0.5, label='e$^\mathregular{-}$')
ax.plot(temp_array[ 0: 5], ion_density_sum[ 0: 5], marker='x', linewidth=0.5, label='sum of ions')

ax.plot(temp_array[ 0: 5], He_met_density_table[ 0: 5], marker='d', linewidth=0.5, label='He*')

ax.set_yticks([0.0E17, 0.5E17, 1E17, 1.5E17, 2E17, 2.5E17, 3E17, 3.5E17], [0.0, 0.5, 1.0, 1.5, 2.0, 2.5, 3.0, 3.5])
ax.set_xticklabels([])
ax.set_ylim([-1E16, 3.6E17])
ax.set_title('He:N$_\mathregular{2}$ fraction 1000:0.5')



# Second plot, #1000:1
ax = axarr[1]
ax.plot(temp_array[ 5:10], He_p_density_table[ 5:10], marker='D', linewidth=0.5, label='He$^\mathregular{+}$')
ax.plot(temp_array[ 5:10], He2_p_density_table[ 5:10], marker='D', linewidth=0.5, label='He$_\mathregular{2}^\mathregular{+}$')
ax.plot(temp_array[ 5:10], N2_p_density_table[ 5:10], marker='D', linewidth=0.5, label='N$_\mathregular{2}^\mathregular{+}$')
ax.plot(temp_array[ 5:10], N4_p_density_table[ 5:10], marker='D', linewidth=0.5, label='N$_\mathregular{4}^\mathregular{+}$')

ax.plot(temp_array[ 0: 5], electron_density_table[ 5:10], marker='|', linewidth=0.5, label='e$^\mathregular{-}$')
ax.plot(temp_array[ 0: 5], ion_density_sum[ 5:10], marker='x', linewidth=0.5, label='sum of ions')

ax.plot(temp_array[ 0: 5], He_met_density_table[ 5:10], marker='d', linewidth=0.5, label='He*')

ax.set_yticks([0.0E17, 0.2E17, 0.4E17, 0.6E17, 0.8E17, 1E17, 1.2E17, 1.4E17], [0.0, 0.2, 0.4, 0.6, 0.8, 1.0, 1.2, 1.4])
ax.set_xticklabels([])
ax.set_title('He:N$_\mathregular{2}$ fraction 1000:1')


# Third plot, #1000:2
ax = axarr[2]
ax.plot(temp_array[10:15], He_p_density_table[10:15], marker='P', linewidth=0.5, label='He$^\mathregular{+}$')
ax.plot(temp_array[10:15], He2_p_density_table[10:15], marker='P', linewidth=0.5, label='He$_\mathregular{2}^\mathregular{+}$')
ax.plot(temp_array[10:15], N2_p_density_table[10:15], marker='P', linewidth=0.5, label='N$_\mathregular{2}^\mathregular{+}$')
ax.plot(temp_array[10:15], N4_p_density_table[10:15], marker='P', linewidth=0.5, label='N$_\mathregular{4}^\mathregular{+}$')

ax.plot(temp_array[ 0: 5], electron_density_table[10:15], marker='|', linewidth=0.5, label='e$^\mathregular{-}$')
ax.plot(temp_array[ 0: 5], ion_density_sum[10:15], marker='x', linewidth=0.5, label='sum of ions')

ax.plot(temp_array[ 0: 5], He_met_density_table[10:15], marker='d', linewidth=0.5, label='He*')

ax.set_yticks([0.0E17, 0.2E17, 0.4E17, 0.6E17, 0.8E17], [0.0, 0.2, 0.4, 0.6, 0.8])
ax.set_xticks([300, 325, 350, 375, 400],
           labels=['300\n(27)', '325\n(52)', '350\n(77)', '375\n(102)', '400\n(127)'])
ax.set_title('He:N$_\mathregular{2}$ fraction 1000:2')

# Shared settings for all plots
for ax in axarr:
    ax.grid(True, color='gray', linestyle='-', linewidth=0.1)
    ax.tick_params(axis='both', which='major', labelsize=13)
    ax.legend(title='particle type', loc='center left', bbox_to_anchor=(1, 0.5))

# Set xlabel, ylabel, title only once
axarr[2].set_xlabel('background gas temperature / K (~°C)', size=15)  # Only for the last subplot
axarr[1].set_ylabel('average particle density / 10$^{\mathregular{17}}$ m$^{\mathregular{-3}}$', size=15)


if 0:
    plt.savefig(r'..\figures\revision' + r'\fig_10_particle_densities.png', dpi=200, bbox_inches='tight')
    plt.savefig(r'..\figures\revision' + r'\fig_10_particle_densities.eps', dpi=200, bbox_inches='tight')

plt.show()
plt.close()









































'''
plt.figure(dpi=200)

if 0:#1000:0.5
    plt.plot(temp_array[ 0: 5], He_p_density_table[ 0: 5], marker='s', linewidth=0.5, label='He$^+$')
    plt.plot(temp_array[ 0: 5], He2_p_density_table[ 0: 5], marker='s', linewidth=0.5, label='He$_2^+$')
    plt.plot(temp_array[ 0: 5], N2_p_density_table[ 0: 5], marker='s', linewidth=0.5, label='N$_2^+$')
    plt.plot(temp_array[ 0: 5], N4_p_density_table[ 0: 5], marker='s', linewidth=0.5, label='N$_4^+$')
    
    plt.plot(temp_array[ 0: 5], electron_density_table[ 0: 5], marker='|', linewidth=0.5, label='e-')
    plt.plot(temp_array[ 0: 5], ion_density_sum[ 0: 5], marker='x', linewidth=0.5, label='sum of Ions')
    
    plt.plot(temp_array[ 0: 5], He_met_density_table[ 0: 5], marker='d', linewidth=0.5, label='He*')
    
    plt.yticks([0.0E17, 0.5E17, 1E17, 1.5E17, 2E17, 2.5E17, 3E17], [0.0, 0.5, 1.0, 1.5, 2.0, 2.5, 3.0])
    


if 0:#1000:1
    plt.plot(temp_array[ 5:10], He_p_density_table[ 5:10], marker='D', linewidth=0.5, label='He$^+$')
    plt.plot(temp_array[ 5:10], He2_p_density_table[ 5:10], marker='D', linewidth=0.5, label='He$_2^+$')
    plt.plot(temp_array[ 5:10], N2_p_density_table[ 5:10], marker='D', linewidth=0.5, label='N$_2^+$')
    plt.plot(temp_array[ 5:10], N4_p_density_table[ 5:10], marker='D', linewidth=0.5, label='N$_4^+$')

    plt.plot(temp_array[ 0: 5], electron_density_table[ 5:10], marker='|', linewidth=0.5, label='e-')
    plt.plot(temp_array[ 0: 5], ion_density_sum[ 5:10], marker='x', linewidth=0.5, label='sum of Ions')
    
    plt.plot(temp_array[ 0: 5], He_met_density_table[ 5:10], marker='d', linewidth=0.5, label='He*')
    
    plt.yticks([0.0E17, 0.2E17, 0.4E17, 0.6E17, 0.8E17, 1E17, 1.2E17, 1.4E17], [0.0, 0.2, 0.4, 0.6, 0.8, 1.0, 1.2, 1.4])



if 0:#1000:2
    plt.plot(temp_array[10:15], He_p_density_table[10:15], marker='P', linewidth=0.5, label='He$^+$')
    plt.plot(temp_array[10:15], He2_p_density_table[10:15], marker='P', linewidth=0.5, label='He$_2^+$')
    plt.plot(temp_array[10:15], N2_p_density_table[10:15], marker='P', linewidth=0.5, label='N$_2^+$')
    plt.plot(temp_array[10:15], N4_p_density_table[10:15], marker='P', linewidth=0.5, label='N$_4^+$')
    
    plt.plot(temp_array[ 0: 5], electron_density_table[10:15], marker='|', linewidth=0.5, label='e-')
    plt.plot(temp_array[ 0: 5], ion_density_sum[10:15], marker='x', linewidth=0.5, label='sum of Ions')
    
    plt.plot(temp_array[ 0: 5], He_met_density_table[10:15], marker='d', linewidth=0.5, label='He*')
    
    plt.yticks([0.0E17, 0.2E17, 0.4E17, 0.6E17, 0.8E17], [0.0, 0.2, 0.4, 0.6, 0.8])



plt.grid(True, color='gray', linestyle='-', linewidth=0.1)

plt.xticks([300, 325, 350, 375, 400],
           labels=['300\n(27)', '325\n(52)', '350\n(77)', '375\n(102)', '400\n(127)'])

plt. axhline(y=0.8E17, color='b', linestyle='--', linewidth=1)
#plt.ylim([-1E16, 1.2E17])
#plt.ylim([-1E16, 2E17])
plt.tick_params(axis='both', which='major', labelsize=13)
#bottom, top = plt.ylim()

plt.xlabel('background gas temperature / K (~°C)', size=15)
plt.ylabel('average particle density / 10$^{\mathregular{17}}$ m$^{\mathregular{-3}}$', size=15)
plt.title('He:N$_2$ fraction 1000:2')


plt.legend(title='particle type', loc='center left', bbox_to_anchor=(1, 0.5))

if 0:
    plt.savefig(r'..\figures\revision' + r'\avg_particle_densities_1k_2.png', dpi=200, bbox_inches='tight')
    plt.savefig(r'..\figures\revision' + r'\avg_particle_densities_1k_2.eps', dpi=200, bbox_inches='tight')

plt.show()
plt.close()
'''
