# -*- coding: utf-8 -*-
'''
Author: David A. Schulenberg
Email: schulenberg@aept.rub.de

This script is part of the research paper "Mode transition induced by gas heating along
the discharge channel in capacitively coupled atmospheric pressure micro plasma jets"

This script is licensed under the MIT License.
See the file "LICENSE.txt" for the full license text.
'''

import re
import numpy as np
import matplotlib.pyplot as plt


plt.rcParams['font.family'] = 'Times New Roman'

def buildpath(foldername, filename):
    base_path = r'..\data_upgraded\fig_11'
    full_path = base_path + '\\' + foldername + '\\' + filename
    return full_path



def readfile(file_path):
    values = []
    data_lines = []
    with open(file_path, "r") as file:
        all_lines = file.readlines()
        data_lines.append(all_lines[4]) #helium_proc_4
        data_lines.append(all_lines[30]) #nitrogen_proc_29
        data_lines.append(all_lines[34]) #Penning ionization
               
        for line in data_lines:
            # Split the line into characters and number using regex:
            # first group matches anything (.*?)
            # second group captures a floating point number, also written in scientific notation:
            match = re.match(r'^(.*?)([-+]?[0-9]*\.?[0-9]+([eE][-+]?[0-9]+)?)$', line.strip())
            if match:
                values.append(float(match.group(2)))
                    
    return values


#list of folders
list_of_folders_1 = ['david_1f_600vpp_05n2_300k',
                     'david_1f_600vpp_05n2_325k',
                     'david_1f_600vpp_05n2_350k',
                     'david_1f_600vpp_05n2_375k',
                     'david_1f_600vpp_05n2_400k', # up tp here 0.5 sccm N2
                     'david_1f_600vpp_1n2_300k',
                     'david_1f_600vpp_1n2_325k',
                     'david_1f_600vpp_1n2_350k',
                     'david_1f_600vpp_1n2_375k',
                     'david_1f_600vpp_1n2_400k', # up tp here 1 sccm N2
                     'david_1f_600vpp_2n2_300k',
                     'david_1f_600vpp_2n2_325k',
                     'david_1f_600vpp_2n2_350k',
                     'david_1f_600vpp_2n2_375k',
                     'david_1f_600vpp_2n2_400k', # up tp here 2 sccm N2
                     ]


temp_array = np.array([300,325,350,375,400])

#create list of full paths
weights = []
sim_times = []
filepaths = []
for folder in list_of_folders_1:
    filepaths.append(buildpath(folder, 'collstat.doc'))
    
    with open(buildpath(folder, 'weight'), 'r') as file:
        weight = file.read().strip()
    weights.append(float(weight))
    
    with open(buildpath(folder, 'time.dat'), 'r') as file:
        time = file.readlines()[1].strip()
    sim_times.append(float(time))
    

#read files from all the folders
tables = []
for path in filepaths:
    tables.append(readfile(path))
    #break

tables = np.array(tables)
weights = np.array(weights)
sim_times = np.array(sim_times)

#weight = 1
volume = 1 
#sim_time = 1
tables[:,0] = tables[:,0] * weights / 1E-7 / sim_times
tables[:,1] = tables[:,1] * weights / 1E-7 / sim_times
tables[:,2] = tables[:,2] * weights / 1E-7 / sim_times



plt.figure(dpi=200)

if 1:
    plt.plot(temp_array, tables[  : 5,0], marker='s', linewidth=0.5, color='tab:blue', label='He, electron impact, 1000:0.5')
    plt.plot(temp_array, tables[ 5:10,0], marker='D', linewidth=0.5, color='tab:blue', label='He, electron impact, 1000: 1')
    plt.plot(temp_array, tables[10:15,0], marker='P', linewidth=0.5, color='tab:blue', label='He, electron impact, 1000: 2')
    
    plt.plot(temp_array, tables[  : 5,1], marker='s', linewidth=0.5, color='tab:orange', label='N$_\mathregular{2}$, electron impact, 1000:0.5')
    plt.plot(temp_array, tables[ 5:10,1], marker='D', linewidth=0.5, color='tab:orange', label='N$_\mathregular{2}$, electron impact, 1000: 1')
    plt.plot(temp_array, tables[10:15,1], marker='P', linewidth=0.5, color='tab:orange', label='N$_\mathregular{2}$, electron impact, 1000: 2')


if 1:
    plt.plot(temp_array, tables[  : 5,2], marker='s', linewidth=0.5, color='tab:green', label='N$_\mathregular{2}$, Penning, 1000:0.5')
    plt.plot(temp_array, tables[ 5:10,2], marker='D', linewidth=0.5, color='tab:green', label='N$_\mathregular{2}$, Penning, 1000: 1')
    plt.plot(temp_array, tables[10:15,2], marker='P', linewidth=0.5, color='tab:green', label='N$_\mathregular{2}$, Penning, 1000: 2')



plt.grid(True, color='gray', linestyle='-', linewidth=0.1)
plt.xticks([300, 325, 350, 375, 400],
           labels=['300\n(27)', '325\n(52)', '350\n(77)', '375\n(102)', '400\n(127)'])

plt.tick_params(axis='both', which='major', labelsize=13)
bottom, top = plt.ylim()

plt.xlabel('background gas temperature / K (~°C)', size=15)
plt.ylabel('ionization rate / m$^\mathregular{-3}$s$^\mathregular{-1}$', size=15)
plt.yscale('log')

plt.legend(title='ionized particle, ionization process, He:N$_\mathregular{2}$ fraction', loc='lower center', bbox_to_anchor=(0.5, 1))
plt.gca().set_aspect(40)

if 0:
    plt.savefig(r'..\figures\revision' + r'\fig_11_ionization_rates.png', dpi=200, bbox_inches='tight')
    plt.savefig(r'..\figures\revision' + r'\fig_11_ionization_rates.eps', dpi=200, bbox_inches='tight')


plt.show()
plt.close()

