# -*- coding: utf-8 -*-
'''
Author: David A. Schulenberg
Email: schulenberg@aept.rub.de

This script is part of the research paper "Mode transition induced by gas heating along
the discharge channel in capacitively coupled atmospheric pressure micro plasma jets"

This script is licensed under the MIT License.
See the file "LICENSE.txt" for the full license text.
'''


import numpy as np
import matplotlib.pyplot as plt

import filehandling as fh

plt.rcParams['font.family'] = 'Times New Roman'


def readavgtemps(array_of_sets):
    sets_temperatures = []
    sets_standarddevs = []
    for setnr in array_of_sets:
        searchstring = ''.join(['0', str(setnr)]) #adds a leading 0 to the set number
        sets_paths = fh.findfilepaths(r'..\data_upgraded\fig_3', searchstring, '?', False)
        setnr_and_sets = np.vstack([np.ones(len(sets_paths))*setnr, np.array(sets_paths)])
        
        set_temperatures = []
        set_stddevs = []
        
        for path in setnr_and_sets[1]:
            temperaturedata = np.loadtxt(path)
            set_temperatures.append(temperaturedata[0])
            set_stddevs.append(temperaturedata[1])
        
        set_mean_temperature = np.average(set_temperatures)
        set_total_stddev = np.sqrt(np.sum(np.square(np.array(set_stddevs))))
        
        sets_temperatures.append([setnr, set_mean_temperature])
        sets_standarddevs.append([set_total_stddev])
        
    return np.hstack([sets_temperatures, sets_standarddevs])



#################################################################################################
##################### Reading data into plottable arrays ########################################
#################################################################################################
mean_temperatures = []
standarddeviations = []

# one jet operating condition for every of the 3 arrays of sets
# there are 4 temperature evaluation files per set, so 24 per array of sets
sets_conditions_1k1  = np.array([15,16,17,18,19,20]) #1000: 1  He:N2
sets_conditions_1kp5 = np.array([22,23,24,25,26,27]) #1000:0.5 He:N2
sets_conditions_1k2  = np.array([29,30,31,32,33,34]) #1000: 2  He:N2
       
sets_1_temperatures = readavgtemps(sets_conditions_1k1)
sets_2_temperatures = readavgtemps(sets_conditions_1kp5)
sets_3_temperatures = readavgtemps(sets_conditions_1k2)

z_position = np.linspace(1.75, 28.25, num=6)*-1



#################################################################################################
################################################### Plot ########################################
#################################################################################################
plt.figure(dpi=200)
plt.errorbar(z_position-0.5, sets_2_temperatures[:,1], yerr=sets_2_temperatures[:,2],
             marker='s', linewidth=0, elinewidth=1, capsize=3, label='1000 : 0.5')
plt.errorbar(z_position, sets_1_temperatures[:,1], yerr=sets_1_temperatures[:,2],
             marker='D', linewidth=0, elinewidth=1, capsize=3, label='1000 :  1')
plt.errorbar(z_position+0.5, sets_3_temperatures[:,1], yerr=sets_3_temperatures[:,2],
             marker='P', linewidth=0, elinewidth=1, capsize=3, label='1000 :  2')


plt.xlim([-30.125, 0.125])
plt.ylim([275, 355])
plt.xticks(z_position)
plt.yticks([280,290,300,310,320,330,340,350],
           labels=(['280\n(7)', '290', '300', '310', '320', '330', '340', '(77)\n350']))
plt.xlabel('z-position / mm', size=15)
plt.ylabel('gas temperature / K (~°C)', size=15)
plt.tick_params(axis='both', which='major', labelsize=13)
plt.legend(title='He : N$_\mathregular{2}$ flow / sccm', ncols = 3,
           columnspacing = 1, handletextpad=0, handlelength=1.5,
           loc='center', title_fontsize=12, fontsize=12, bbox_to_anchor=(0.5, 1.1))
plt.grid(True, color='gray', linestyle='-', linewidth=0.1)
plt.gca().set_aspect(0.35)


if 0:
    plt.savefig(r'..\figures' + '\\fig_3_OES_temperatures.png', dpi=200, bbox_inches='tight')
    plt.savefig(r'..\figures' + '\\fig_3_OES_temperatures.eps', dpi=200, bbox_inches='tight')


plt.show()
plt.close()