# -*- coding: utf-8 -*-
'''
Author: David A. Schulenberg
Email: schulenberg@aept.rub.de

This script is part of the research paper "Mode transition induced by gas heating along
the discharge channel in capacitively coupled atmospheric pressure micro plasma jets"

This script is licensed under the MIT License.
See the file "LICENSE.txt" for the full license text.
'''

import re
import os
import numpy as np
import matplotlib.pyplot as plt
from gas_temp_increase_estimation import estimate_tgas_increase

plt.rcParams['font.family'] = 'Times New Roman'

def buildpath(foldername):
    base_path = r'..\data_upgraded\fig_4_fig_5'
    filename = 'elastic_power_transfer.dat'
    full_path = base_path + '\\' + foldername + '\\' + filename
    return full_path


def readpowerfiles(file_path):
    particle_type = []
    values = []
    with open(file_path, 'r') as file:
        next(file) # Skip the first line
        for line in file:
            # Split the line into characters and number using regex:
            # first group matches anything (.*?)
            # second group captures a floating point number, also written in scientific notation:
            match = re.match(r"^(.*?)([-+]?[0-9]*\.?[0-9]+([eE][-+]?[0-9]+)?)$", line.strip())
            if match:
                particle_type.append(match.group(1)[:-3])
                values.append(float(match.group(2)))
                
    values_array = np.array(values)
    file_contents = [particle_type, values_array]
    powertable = [file_path.split(os.sep)[-2], file_contents]
    return powertable


power_density = 1 #J/(s m³)
temp_increase_relative = estimate_tgas_increase(power_density) #K/(W/m³)


#list of folders
list_of_folders_1 = ['david_1f_600vpp_05n2_300k',
                     'david_1f_600vpp_05n2_325k',
                     'david_1f_600vpp_05n2_350k',
                     'david_1f_600vpp_05n2_375k',
                     'david_1f_600vpp_05n2_400k', # up tp here 0.5 sccm N2
                     'david_1f_600vpp_1n2_300k',
                     'david_1f_600vpp_1n2_325k',
                     'david_1f_600vpp_1n2_350k',
                     'david_1f_600vpp_1n2_375k',
                     'david_1f_600vpp_1n2_400k', # up tp here 1 sccm N2
                     'david_1f_600vpp_2n2_300k',
                     'david_1f_600vpp_2n2_325k',
                     'david_1f_600vpp_2n2_350k',
                     'david_1f_600vpp_2n2_375k',
                     'david_1f_600vpp_2n2_400k', # up tp here 2 sccm N2
                     ]


#create list of full paths
filepaths = []
for folder in list_of_folders_1:
    filepaths.append(buildpath(folder))
    

#read files from all the folders
powertables = []
for path in filepaths:
    powertables.append(readpowerfiles(path))


#extract and order electron power values
electron_power_table = []
temp_list = []
for table in powertables:
    foldername = table[0]
    electron_power = table[1][1][0]
    electron_power_table.append(electron_power)
    temp_list.append(int(foldername[-4:-1]))

electron_power_table = np.array(electron_power_table)
temp_inc_array_electrons = np.array(electron_power_table)*temp_increase_relative


#extract and order ion power values
ion_power_table = []
temp_list = []
for table in powertables:
    foldername = table[0]
    ion_powers = table[1][1][1:]
    power_sum = sum(ion_powers)
    ion_power_table.append(power_sum)
    temp_list.append(int(foldername[-4:-1]))

ion_power_table = np.array(ion_power_table)
temp_inc_array_ions = np.array(ion_power_table)*temp_increase_relative
temp_array = np.array(temp_list)



#################################################################################################
################# Plot electron and ion elastic collision powers ################################
#################################################################################################
if 1:
    plt.figure(dpi=200)

    plt.plot(temp_array[ 0: 5], electron_power_table[ 0: 5]/1E6, marker='s', linewidth=0.5, color='tab:red', label='electrons, 1000 : 0.5')
    plt.plot(temp_array[ 5:10], electron_power_table[ 5:10]/1E6, marker='D', linewidth=0.5, color='tab:red', label='electrons, 1000 :  1')
    plt.plot(temp_array[10:15], electron_power_table[10:15]/1E6, marker='P', linewidth=0.5, color='tab:red', label='electrons, 1000 :  2')
    
    plt.plot(temp_array[ 0: 5], ion_power_table[ 0: 5]/1E6, marker='s', linewidth=0.5, color='tab:cyan', label='ions, 1000 : 0.5')
    plt.plot(temp_array[ 5:10], ion_power_table[ 5:10]/1E6, marker='D', linewidth=0.5, color='tab:cyan', label='ions, 1000 :  1')
    plt.plot(temp_array[10:15], ion_power_table[10:15]/1E6, marker='P', linewidth=0.5, color='tab:cyan', label='ions, 1000 :  2')
    
    
    plt.grid(True, color='gray', linestyle='-', linewidth=0.1)
    plt.xticks([300, 325, 350, 375, 400],
               labels=['300\n(27)', '325\n(52)', '350\n(77)', '375\n(102)', '400\n(127)'])
    #plt.yticks([0,5,10,15])
    plt.ylim([0, 12])
    plt.tick_params(axis='both', which='major', labelsize=13)
    
    plt.xlabel('background gas temperature / K (~°C)', size=15)
    plt.ylabel('elastic collision\npower density / MWm$^{\mathregular{-3}}$ ', size=15)
    
    plt.legend(title='He : N$_\mathregular{2}$ fraction',
               ncols=2, columnspacing=1, handletextpad=0, handlelength=1.5,
               loc='center', title_fontsize=12, fontsize=12, bbox_to_anchor=(0.5, 1.2))
    plt.gca().set_aspect(8)
    
    if 0:
        plt.savefig(r'..\figures' + '\\fig_4_PIC_power_density.png', dpi=200, bbox_inches='tight')
        plt.savefig(r'..\figures' + '\\fig_4_PIC_power_density.eps', dpi=200, bbox_inches='tight')

    plt.show()
    plt.close()




#################################################################################################
###### Plot estimated temperature increase based on charged particle power densities ############
#################################################################################################
if 1:
    plt.figure(dpi=200)
    
    plt.plot(temp_array[ 0: 5], (electron_power_table[ 0: 5]+ion_power_table[ 0: 5])*temp_increase_relative, marker='s', linewidth=0.5, label='1000 : 0.5')
    plt.plot(temp_array[ 5:10], (electron_power_table[ 5:10]+ion_power_table[ 5:10])*temp_increase_relative, marker='D', linewidth=0.5, label='1000 :  1')
    plt.plot(temp_array[10:15], (electron_power_table[10:15]+ion_power_table[10:15])*temp_increase_relative, marker='P', linewidth=0.5, label='1000 :  2')


    plt.grid(True, color='gray', linestyle='-', linewidth=0.1)
    plt.xticks([300, 325, 350, 375, 400],
               labels=['300\n(27)', '325\n(52)', '350\n(77)', '375\n(102)', '400\n(127)'])
    plt.ylim([0, 30])
    plt.tick_params(axis='both', which='major', labelsize=13)
    
    plt.xlabel('background gas temperature / K (~°C)', size=15)
    plt.ylabel('estimated increase of T$_\mathregular{gas}$ / K', size=15)

    plt.legend(title='He : N$_\mathregular{2}$ fraction',
               ncols=3, columnspacing=1, handletextpad=0, handlelength=1.5,
               loc='center', title_fontsize=12, fontsize=12, bbox_to_anchor=(0.5, 1.1))
    plt.gca().set_aspect(3)
    
    if 0:
        plt.savefig(r'..\figures' + '\\fig_5_PIC_temp_increase.png', dpi=200, bbox_inches='tight')
        plt.savefig(r'..\figures' + '\\fig_5_PIC_temp_increase.eps', dpi=200, bbox_inches='tight')

    plt.show()
    plt.close()
    
