# -*- coding: utf-8 -*-
'''
Author: David A. Schulenberg
Email: schulenberg@aept.rub.de

This script is part of the research paper "Mode transition induced by gas heating along
the discharge channel in capacitively coupled atmospheric pressure micro plasma jets"

This script is licensed under the MIT License.
See the file "LICENSE.txt" for the full license text.
'''

import os
import numpy as np
import matplotlib.pyplot as plt
from matplotlib.ticker import AutoMinorLocator

import filehandling as fh

exc_files = fh.findfilepaths(r'..\data_upgraded\fig_6', 'exc.dat', '?', False)

XTdata = []
filenames = []
maxdatavalue = 0

for exc_set in exc_files:
    startframe = 12
    period = 37
    data = np.loadtxt(exc_set)[:, startframe:startframe+period]
    data = data/np.max(data)
    XTdata.append(data)
    filenames.append(os.path.basename(exc_set))

XTdata.reverse()


numofsubplots = len(XTdata)

figure, axlist = plt.subplots(1, numofsubplots, dpi=200)

figure.set_size_inches(7.5, 1)
x_ticks=np.array([0,36])
y_ticks=np.array([0,204])

for ax in axlist:
    ax.set_xticks(x_ticks)
    ax.set_yticks(y_ticks)
    ax.set_xticklabels([0, 0.5])
    ax.set_yticklabels([0, 1])

    ax.set_xticks([18], minor=True)
    ax.set_yticks([102], minor=True)
    
plt.setp(axlist, ylabel='distance from groun-\nded electrode / mm')

titlelist = ['337 K\n-1.75\u2009mm',
             '340 K\n-7.05\u2009mm',
             '337 K\n-12.35\u2009mm',
             '328 K\n-17.65\u2009mm',
             '317 K\n-22.95\u2009mm',
             '302 K\n-28.25\u2009mm']
titlelist.reverse()

lvs = np.linspace(0, 1, num=15)
for dataindex, axisnr in enumerate(axlist):
    plt1 = axisnr.contourf(XTdata[dataindex], levels = lvs, cmap='seismic')
    axisnr.set_title(str(titlelist[dataindex]), y=1.05, size=10)
    if dataindex > 0:
        axisnr.set_ylabel('')  # Empty the y-axis label
        axisnr.set_yticklabels([])

figure.subplots_adjust(wspace=0.25, hspace=0.1)


cbar_ax = figure.add_axes([0.25, 1.35, 0.5, 0.05]) # [left, bottom, width, height]
cbar = figure.colorbar(plt1, cax=cbar_ax, orientation='horizontal')
cbar.set_ticks([0, 0.25, 0.5, 0.75, 1])
cbar.set_label('excitation rate from the ground state to He I (3s)$^\mathregular{3}$S$_\mathregular{1}$ / a.u.', size=10, labelpad=1)
cbar.ax.xaxis.set_label_coords(0.5, 9)
cbar.ax.tick_params(labelbottom=False, labeltop=True, direction='in', length=7, width=1, pad=6)

plt.figtext(0.5, -0.25, 'time / T$_\mathregular{RF}$', ha='center', size=10)

    
if 0:
    plt.savefig(r'..\figures' + '\\fig_6_PROES_horizontal.png', dpi=200, bbox_inches='tight')
    plt.savefig(r'..\figures' + '\\fig_6_PROES_horizontal.eps', dpi=200, bbox_inches='tight')

plt.show()
plt.close()
 








