# -*- coding: utf-8 -*-
'''
Author: David A. Schulenberg
Email: schulenberg@aept.rub.de

This script is part of the research paper "Mode transition induced by gas heating along
the discharge channel in capacitively coupled atmospheric pressure micro plasma jets"

This script is licensed under the MIT License.
See the file "LICENSE.txt" for the full license text.
'''

import os
import numpy as np
import matplotlib.pyplot as plt
import scipy.ndimage as ndi
import filehandling as fh

from skimage.measure import find_contours
from skimage.segmentation import watershed
from skimage.feature import peak_local_max



#################################################################################################
###################################### Helper functions  ########################################
#################################################################################################

def fastplot(contourdata, plotlinex, plotliney, plotpointsx, plotpointsy, title):
    #This function is used to plot things throughout the script requiring only a one-liner
    plt.figure(dpi=200)
    plt.contourf(contourdata, 20)
    plt.plot(plotlinex, plotliney, color='red', lw=3, markersize=0)
    plt.plot(plotpointsx, plotpointsy, color='red', lw=0, marker='.', markersize=10)
        
    plt.title(title)
    plt.ylabel("distance / pixels")
    plt.xlabel("time / frames")
    plt.colorbar()
    plt.show()
    plt.close()



def read_ROIs(filepath, roll, start, bottom, top, vertical, period, plot_data):
    #Reads and returns rectangular patches of data defined by the start, bottom, top, and vertical coordinates.
    #The vertical coordinates are a list, so several rectangles next to each other are extracted from the data.
    ROIs = []
    for left in vertical:
        right = left + int(period/2)
        ROI_frame = np.array([[top,left],[bottom,left],[bottom,right],[top,right],[top,left]])
        XTdata = np.loadtxt(filepath)[:, start:]
        for _ in range(roll):
            XTdata = np.roll(XTdata, -1, axis=1)
        ROI = XTdata[bottom:top, left:right]
        ROIs.append(ROI)
        if plot_data: fastplot(ndi.gaussian_filter(XTdata, sigma=2), ROI_frame[0:, 1], ROI_frame[0:, 0], 0, 0, os.path.basename(filepath))
        #if plot_data: fastplot(XTdata, ROI_frame[0:, 1], ROI_frame[0:, 0], 0, 0, os.path.basename(filepath))
        
    return np.array(ROIs)



def find_exactly_x_peaks(x, data, footprint_ratio, smooth_data, plot_data, plottitle):
    #This function uses peak_local_max to find peaks in 2D data. It returns the coordinates of the peaks.
    #Initially lots of peaks are found. Then the searach footprint is increased until exactly x peaks are found.
    #If this does not work (more than x peaks in one step, less than x in the next), the function calls itself
    #with an increased footprint ratio (height/widht). This works for my kind of data. Almost certainly,
    #unhandeled edge cases exist for other kinds of similar data.
    if smooth_data: data = ndi.gaussian_filter(data, sigma=2)
    peak_coordinates = []
    i = 1
    while len(peak_coordinates) != x:
        i += 1
        peak_coordinates = peak_local_max(data, footprint=np.ones((round(i*footprint_ratio), i)),)
        #fastplot(data, peak_coordinates[:, 1], peak_coordinates[:, 0], plottitle)
        if len(peak_coordinates) < x:
            print('_______________')
            print(plottitle)
            print('Not exactly %d peaks found in this attempt, footprint ratio: %f' %(x, footprint_ratio))
            peak_coordinates = find_exactly_x_peaks(x, data, footprint_ratio+1, smooth_data, False, '')
            #fastplot(data, peak_coordinates[:, 1], peak_coordinates[:, 0], plottitle)
            break
    if plot_data: fastplot(data, False, False, peak_coordinates[:, 1], peak_coordinates[:, 0], plottitle)
        
    return peak_coordinates
    


def do_watershed(data, seedpoints):
    #Watershed image segmentation. Returns an array the size of the input data in which all the basins found by the 
    #watershed segmentation are labeled using increasing ints starting from 1 
    data = ndi.gaussian_filter(data, sigma=0.5)
    mask = np.zeros(data.shape, dtype=bool)
    mask[tuple(seedpoints.T)] = True
    markers, _ = ndi.label(mask)
    
    return watershed(-data, markers)



def get_threshold_value(data1, percentage):
    #Returns the smallest of the top *percentage* percent of the values in *data1*
    data_flat_1 = np.ndarray.flatten(data1)
    perc1 = int(len(data_flat_1)/(100/percentage))
    ind1 = np.argpartition(data_flat_1, -perc1)[-perc1:]
    
    return np.min(data_flat_1[ind1])
   
    
   
def join_basins(basin_1, basin_2):
    return np.where(basin_1 != 0, basin_1, basin_2)
    


def plot_final_ratios(ratios, reverse_data, saveplot):
    filenames = [filename_and_ratio[0][-35:-16] for filename_and_ratio in all_ratios]
    mean_ratios = [filename_and_ratio[1][0] for filename_and_ratio in all_ratios]
    stddev_ratios = [filename_and_ratio[1][1] for filename_and_ratio in all_ratios]
    
    if reverse_data:
        filenames.reverse()
        mean_ratios.reverse()
        stddev_ratios.reverse()

    plt.figure(dpi=200)
    xticks = range(len(mean_ratios))
    plt.errorbar(xticks, mean_ratios, yerr=stddev_ratios, linestyle='none', marker='o', elinewidth=1.0, capsize=3)
    
    plt.ylim(0.5,1)
    plt.xticks(xticks, filenames, rotation=90)

    plt.xlabel('z-position / mm')
    plt.ylabel('ratio of penning- to omega-peak')
    plt.title('Title');
    if saveplot:
        plt.savefig(filenames[0][-18:-4]+'_ratios.png', dpi=300, bbox_inches='tight')
    plt.show()
    plt.close()
    
    
    
def save_evaluation(data, filenameextention):
    filenames = [part[0][-35:-16] for part in data]
    mean_ratios = [part[1][0] for part in data]
    stddev_ratios = [part[1][1] for part in data]
    
    switch = 1
    with open('ratiolist_'+filenameextention+'.txt', 'w') as f:
        for values in zip(filenames, mean_ratios, stddev_ratios):
            if switch:
                line = '\t'.join(map(str, ['set', 'peak ratio', 'standard deviation']))
                f.write(line)
                f.write('\n')
                switch = 0
            line = '\t'.join(map(str, values))
            f.write(line)
            f.write('\n')
    print('saved')
    



#################################################################################################
########################################## Main Program  ########################################
#################################################################################################
path = r'..\data_upgraded\fig_9\0p5N2'
#path = r'..\data_upgraded\fig_9\1N2'
#path = r'..\data_upgraded\fig_9\2N2'


exc_file_paths = fh.findfilepaths(path, 'HE_706NM_tx.dat', '?', False)


#define ROI borders in units of datapoints
start = 0
bottom = 0
top = 300
vertical = [0, 100]
period = 200

thresholds = np.linspace(7,1,7)


all_ratios = []
for file in exc_file_paths:
    ROIs_data = read_ROIs(file, 50, start, bottom, top, vertical, period, plot_data=True)
    
    ratios = []
    for ROI in ROIs_data:
        ROI = ndi.gaussian_filter(ROI, sigma=2)
        peak_coordinates = find_exactly_x_peaks(2, ROI, 6, True, True, file[-35:-16])
        basin_number = do_watershed(ROI, peak_coordinates)
        border = find_contours(basin_number, 1)[0]
        
        basin_1 = np.copy(ROI)
        basin_2 = np.copy(ROI)
        basin_1[basin_number == 2] = 0
        basin_2[basin_number == 1] = 0
        
        for threshold in thresholds:
            if 0: #use the smaller peak as threshold reference
                if np.max(basin_1) > np.max(basin_2): ref = basin_2
                else: ref = basin_1
                basin_1[basin_1 < get_threshold_value(ref, threshold)] = 0
                basin_2[basin_2 < get_threshold_value(ref, threshold)] = 0
                
            if 1:#apply thresholds for each peak individually
                basin_1[basin_1 < get_threshold_value(basin_1, threshold)] = 0
                basin_2[basin_2 < get_threshold_value(basin_2, threshold)] = 0
            
            ratio = np.mean(basin_2[basin_2 != 0]) / np.mean(basin_1[basin_1 != 0])
            if basin_number[100,0] == 2: ratio = 1/ratio
            ratios.append(ratio)
            
            if 1:
                fastplot(join_basins(basin_1,basin_2), border[0:,1], border[0:,0], 0, 0, file[-18:]+f' {threshold}')
            
    ratio_mean_stddev = [np.mean(ratios), np.std(ratios)]
    filename_and_ratio = [file, ratio_mean_stddev]
    all_ratios.append(filename_and_ratio)
    
    print('ratio mean: %f' % ratio_mean_stddev[0])
    print('ratio stddev:  %f' % ratio_mean_stddev[1])
    print('')
    print('')

plot_final_ratios(all_ratios, False, False)

os.chdir(path)
save_evaluation(all_ratios, path[-3:])
